from .investigator import Investigator
from .nbhandlers import commands

__type__ = "plugin"
__charactor__ = Investigator
__name__ = "coc"
__cname__ = "调查员"
__nbhandler__ = nbhandlers
__nbcommands__ = commands